@@ -1,23 +1,22 @@
 from __future__ import annotations
 
+import ast
+import contextlib
+import functools
 import importlib.util
 import os
 import re
 import sys
-import sysconfig
 import types
 import urllib.parse
-import warnings
-from contextlib import contextmanager
 from fnmatch import fnmatchcase
 from pathlib import Path
-from typing import Callable, Generator, Iterable, Match
+from typing import Any, Callable, Generator, Iterable, Match
 
-from pdm.backend._vendor.packaging import tags
 from pdm.backend._vendor.packaging.markers import Marker
 from pdm.backend._vendor.packaging.requirements import Requirement
 from pdm.backend._vendor.packaging.version import InvalidVersion, Version
-from pdm.backend.macosx_platform import calculate_macosx_platform_tag
+from pdm.backend.exceptions import ConfigError
 
 
 def safe_version(version: str) -> str:
@@ -106,80 +105,12 @@ def find_packages_iter(
             dirs.append(dir)
 
 
-@contextmanager
-def cd(path: str) -> Generator[None, None, None]:
-    _old_cwd = os.getcwd()
-    os.chdir(path)
-    try:
-        yield
-    finally:
-        os.chdir(_old_cwd)
-
-
 def normalize_path(filename: str | Path) -> str:
     """Normalize a file/dir name for comparison purposes"""
     filename = os.path.abspath(filename) if sys.platform == "cygwin" else filename
     return os.path.normcase(os.path.realpath(os.path.normpath(filename)))
 
 
-def get_platform(build_dir: str | Path) -> str:
-    """Return our platform name 'win32', 'linux_x86_64'"""
-    result = sysconfig.get_platform()
-    if result.startswith("macosx") and os.path.exists(build_dir):
-        result = calculate_macosx_platform_tag(build_dir, result)
-    if result in ("linux_x86_64", "linux-x86_64") and sys.maxsize == 2147483647:
-        # pip pull request #3497
-        result = "linux_i686"
-    return result
-
-
-def get_flag(
-    var: str, fallback: bool, expected: bool = True, warn: bool = True
-) -> bool:
-    """Use a fallback value for determining SOABI flags if the needed config
-    var is unset or unavailable."""
-    val = sysconfig.get_config_var(var)
-    if val is None:
-        if warn:
-            warnings.warn(
-                "Config variable '{}' is unset, Python ABI tag may "
-                "be incorrect".format(var),
-                RuntimeWarning,
-                2,
-            )
-        return fallback
-    return val == expected
-
-
-def get_abi_tag() -> str | None:
-    """Return the ABI tag based on SOABI (if available) or emulate SOABI
-    (CPython 2, PyPy)."""
-    soabi = sysconfig.get_config_var("SOABI")
-    impl = tags.interpreter_name()
-    is_cpython = impl == "cp"
-    if not soabi and impl in ("cp", "pp") and hasattr(sys, "maxunicode"):
-        d = ""
-        m = ""
-        u = ""
-        if get_flag("Py_DEBUG", hasattr(sys, "gettotalrefcount"), warn=is_cpython):
-            d = "d"
-        if sys.version_info < (3, 8) and get_flag(
-            "WITH_PYMALLOC", is_cpython, warn=is_cpython
-        ):
-            m = "m"
-        if sys.version_info < (3, 3) and get_flag(
-            "Py_UNICODE_SIZE", sys.maxunicode == 0x10FFFF, expected=4, warn=is_cpython
-        ):
-            u = "u"
-        return f"{impl}{tags.interpreter_version()}{d}{m}{u}"
-    elif soabi and soabi.startswith("cpython-"):
-        return "cp" + soabi.split("-")[1]
-    elif soabi:
-        return soabi.replace(".", "_").replace("-", "_")
-    else:
-        return None
-
-
 def is_relative_path(target: Path, other: Path) -> bool:
     try:
         target.relative_to(other)
@@ -210,14 +141,18 @@ def expand_vars(line: str, root: str) -> str:
 
 
 def import_module_at_path(
-    src_path: str | Path, module_name: str = "_local"
+    src_path: str | Path, module_name: str = "_local", context: Path | None = None
 ) -> types.ModuleType:
     """Import a module from a given path."""
     spec = importlib.util.spec_from_file_location(module_name, src_path)
     if spec is None:
         raise ValueError(f"Could not import module {module_name} from {src_path}")
+    if context is not None:
+        sys.path.insert(0, str(context.absolute()))
     module = importlib.util.module_from_spec(spec)
     spec.loader.exec_module(module)  # type: ignore
+    if context is not None:
+        sys.path.pop(0)
     return module
 
 
@@ -234,3 +169,49 @@ def normalize_file_permissions(st_mode: int) -> int:
         new_mode |= 0o111  # Executable: 644 -> 755
 
     return new_mode
+
+
+@contextlib.contextmanager
+def patch_sys_path(path: str | Path) -> Generator[None]:
+    old_path = sys.path[:]
+    sys.path.insert(0, str(path))
+    try:
+        yield
+    finally:
+        sys.path[:] = old_path
+
+
+_attr_regex = re.compile(r"([\w.]+):([\w.]+)\s*(\([^)]+\))?")
+
+
+def evaluate_module_attribute(
+    expression: str, context: Path | None = None
+) -> tuple[Any, tuple[Any, ...]]:
+    """Evaluate the value of an expression like '<module>:<attribute>'
+
+    Returns:
+        the object and the calling arguments if any
+    """
+    if context is None:
+        cm = contextlib.nullcontext()
+    else:
+        cm = patch_sys_path(context)  # type: ignore[assignment]
+
+    matched = _attr_regex.match(expression)
+    if matched is None:
+        raise ConfigError(
+            "Invalid expression, must be in the format of " "`module:attribute`."
+        )
+    with cm:
+        module = importlib.import_module(matched.group(1))
+        attrs = matched.group(2).split(".")
+        obj: Any = functools.reduce(getattr, attrs, module)
+        args_group = matched.group(3)
+        if args_group:
+            # make tuple
+            args_group = args_group.strip()[:-1] + ",)"
+            args = ast.literal_eval(args_group)
+
+        else:
+            args = ()
+        return obj, args
