@@ -1,6 +1,7 @@
 import email
 import os
 import sys
+import tarfile
 import zipfile
 from pathlib import Path
 
@@ -42,6 +43,32 @@ def test_build_single_module(dist: Path) -> None:
     assert "demo_module-0.1.0.dist-info/licenses/LICENSE" in zip_names
 
 
+@pytest.mark.parametrize("name", ["demo-module"])
+def test_build_single_module_with_build_number(dist: Path) -> None:
+    build_number = "20231241"
+    wheel_name = api.build_wheel(
+        dist.as_posix(),
+        config_settings={"--build-number": build_number},
+    )
+    assert wheel_name == f"demo_module-0.1.0-{build_number}-py3-none-any.whl"
+    with zipfile.ZipFile(dist / wheel_name) as zf:
+        wheel_metadata = email.message_from_bytes(
+            zf.read("demo_module-0.1.0.dist-info/WHEEL")
+        )
+        assert wheel_metadata["Build"] == build_number
+
+
+@pytest.mark.parametrize("name", ["demo-module"])
+def test_build_single_module_without_build_number(dist: Path) -> None:
+    wheel_name = api.build_wheel(dist.as_posix())
+    assert wheel_name == "demo_module-0.1.0-py3-none-any.whl"
+    with zipfile.ZipFile(dist / wheel_name) as zf:
+        wheel_metadata = email.message_from_bytes(
+            zf.read("demo_module-0.1.0.dist-info/WHEEL")
+        )
+        assert "Build" not in wheel_metadata
+
+
 @pytest.mark.parametrize("name", ["demo-package"])
 def test_build_package(dist: Path) -> None:
     wheel_name = api.build_wheel(dist.as_posix())
@@ -163,6 +190,27 @@ def test_build_explicit_package_dir(dist: Path) -> None:
     assert "my_package/data.json" in zip_names
 
 
+@pytest.mark.parametrize("name", ["demo-metadata-test"])
+def test_demo_metadata_test__sdist__pkg_info(
+    dist: Path, name: str, tmp_path: Path
+) -> None:
+    filename = api.build_sdist(dist.as_posix())
+    with tarfile.open(dist / filename) as archive:
+        archive.extractall(tmp_path)
+    pkg_info_path = next(tmp_path.rglob("PKG-INFO"))
+    parsed_pkg_info = email.message_from_bytes(pkg_info_path.read_bytes())
+    assert dict(parsed_pkg_info) == {
+        "Author-Email": '"Corporation, Inc." <corporation@example.com>, Example '
+        "<example@example.com>",
+        "Description-Content-Type": "text/markdown",
+        "License-Expression": "MIT",
+        "Metadata-Version": "2.4",
+        "Name": name,
+        "Requires-Python": ">=3.8",
+        "Version": "3.2.1",
+    }
+
+
 @pytest.mark.parametrize("name", ["demo-package"])
 def test_prepare_metadata(dist: Path) -> None:
     dist_info = api.prepare_metadata_for_build_wheel(dist.as_posix())
@@ -248,15 +296,15 @@ def test_build_with_cextension_in_src(dist: Path) -> None:
 @pytest.mark.parametrize("name", ["demo-package"])
 def test_build_editable(dist: Path, fixture_project: Path) -> None:
     wheel_name = api.build_editable(dist.as_posix())
-    assert api.get_requires_for_build_editable() == ["editables"]
+    assert api.get_requires_for_build_editable() == []
     with zipfile.ZipFile(dist / wheel_name) as zf:
         namelist = zf.namelist()
         assert "demo_package.pth" in namelist
         assert "_editable_impl_demo_package.py" in namelist
-        assert "demo_package-0.1.0+editable.dist-info/licenses/LICENSE" in namelist
+        assert "demo_package-0.1.0.dist-info/licenses/LICENSE" in namelist
 
         metadata = email.message_from_bytes(
-            zf.read("demo_package-0.1.0+editable.dist-info/METADATA")
+            zf.read("demo_package-0.1.0.dist-info/METADATA")
         )
         assert "editables" in metadata.get_all("Requires-Dist", [])
 
@@ -313,7 +361,7 @@ def test_build_editable_pep420(dist: Path, fixture_project: Path) -> None:
         assert "__editables_demo_package.py" not in namelist
 
         metadata = email.message_from_bytes(
-            zf.read("demo_package-0.1.0+editable.dist-info/METADATA")
+            zf.read("demo_package-0.1.0.dist-info/METADATA")
         )
         assert "editables" not in metadata.get_all("Requires-Dist", [])
 
@@ -324,7 +372,7 @@ def test_build_editable_pep420(dist: Path, fixture_project: Path) -> None:
 @pytest.mark.parametrize("name", ["demo-package"])
 def test_prepare_metadata_for_editable(dist: Path) -> None:
     dist_info = api.prepare_metadata_for_build_editable(dist.as_posix())
-    assert dist_info == "demo_package-0.1.0+editable.dist-info"
+    assert dist_info == "demo_package-0.1.0.dist-info"
     with (dist / dist_info / "METADATA").open("rb") as metadata:
         deps = email.message_from_binary_file(metadata).get_all("Requires-Dist")
     assert "editables" in deps
@@ -398,6 +446,19 @@ def test_override_scm_version_via_env_var(
     assert wheel_name == "foo-1.0.0-py3-none-any.whl"
 
 
+@pytest.mark.usefixtures("scm")
+@pytest.mark.parametrize("name", ["demo-using-scm"])
+def test_build_wheel_custom_version_format(fixture_project: Path, dist) -> None:
+    builder = WheelBuilder(fixture_project)
+    builder.config.data.setdefault("tool", {}).setdefault("pdm", {})["version"] = {
+        "source": "scm",
+        "version_format": "version:format_version",
+    }
+    with builder:
+        wheel = builder.build(dist)
+        assert wheel.name == "foo-0.1.0rc0-py3-none-any.whl"
+
+
 @pytest.mark.usefixtures("scm")
 @pytest.mark.parametrize("getter", ["get_version:run", "get_version:run()"])
 @pytest.mark.parametrize("name", ["demo-using-scm"])
@@ -476,3 +537,18 @@ def test_clean_not_called_if_config_settings_exist(
             assert not os.path.exists(test_file)
         else:
             assert os.path.exists(test_file)
+
+
+@pytest.mark.parametrize("name", ["demo-licenses"])
+def test_build_wheel_with_license_file(fixture_project: Path, dist: Path) -> None:
+    wheel_name = api.build_wheel(dist.as_posix())
+    sdist_name = api.build_sdist(dist.as_posix())
+
+    tar_names = get_tarball_names(dist / sdist_name)
+    licenses = ["LICENSE", "licenses/LICENSE.MIT.md", "licenses/LICENSE.APACHE.md"]
+    for file in licenses:
+        assert f"demo_module-0.1.0/{file}" in tar_names
+
+    zip_names = get_wheel_names(dist / wheel_name)
+    for file in licenses:
+        assert f"demo_module-0.1.0.dist-info/licenses/{file}" in zip_names
